/*
 resampleimage

   Authors:  Hassan Siddiqui - XMM-SOC
         
   Description: Combines rudi-5 sub-images into a single mosaic
 
   Comments: 
 
  $Id: resampleimage.cc,v 1.2 2000/04/11 09:37:06 hsiddiqu Exp $
*/

/* Sas-specific */
#include "Tracer.h"
#include "errstr.h"
#include "param.h"
#include "Dal.h"

/* Local */
#include "resampleimage.h"

void resampleimage ()
{
  PROC;

  const unsigned factor = 2;

  string image = stringParameter("array");

  errstr << "Accessing " << image << "..."
	 << Sas::message(Msg::App, Msg::Sparse);
  
  DataSet * set = dataSetServer->open(image, Dal::Read);
  
  Array * array = set->array(image);
  
  /* Get size info */
  longVec arraysize = array->size();
  unsigned xsubsize = arraysize[1];
  unsigned ysubsize = arraysize[0];
  
  longVec outsize;
  outsize.push_back(ysubsize*factor);
  outsize.push_back(xsubsize*factor);

  /* Terminate task if array is not rebinned */
  unsigned xwinsize = array->attribute("WINDOWDX")->asInt();
  unsigned ywinsize = array->attribute("WINDOWDY")->asInt();

  if (outsize[1] != xwinsize && outsize[2] != ywinsize)
	{
	  errstr 
	    << "Expected the \'unbinned\' size determined from WINDOWDX, WINDOWDY. (["
	    << xwinsize << "," << ywinsize
	    << "]) to be " << factor
	    << " times the size of array [ "
	    << xsubsize << "," << ysubsize
	    << "]"
	    << Sas::error("arraySizeInvalid");
	  ALTRETURN;
	}

  /* Create new dataset */
  string outimage = stringParameter("outarray");
  DataSet * outset = dataSetServer->open(outimage, Dal::Create);
  
  /* Copy dataset attributes */
  outset->addAttributes(set);

  /* Create new array */
  Array * outarray = outset->addArray(outimage,
				      Dal::Int32,
				      outsize);

  /* Copy array attributes */
  outarray->addAttributes(array);

  /* Get array data for i/p image */
  ArrayData * arraydata = array->data();
  int32 * data = arraydata->int32Data();

  /* Get array data for o/p image */
  ArrayData * outarraydata = outarray->data();
  int32 * outdata = outarraydata->int32Data();

  /* Iterate though pixels of i/p image */
  for (unsigned x = 0; x < xsubsize; ++x)
    {
      for (unsigned y=0; y < ysubsize; ++y)
	{
	  /* Compute value of output pixels corresponding to
	     the input pixel.
	  */
	  int32 value = data[x+(y*xsubsize)];
	  int32 outvalue = value/(factor*factor);

	  for (unsigned j=0; j<factor; ++j)
	    {
	      for (unsigned k=0; k<factor; ++k)
		{
		  unsigned xpos = x*factor+j;
		  unsigned ypos = y*factor+k;
	  
		  /* Write value to o/p image */
		  errstr 
		    << "Writing a value of "
		    << outvalue
		    << " to position ["
		    << xpos << "," << ypos
		    << "] "
		    << Sas::message(Msg::App, Msg::Verbose);

		  outdata[xpos + (ypos*outsize[1])] = outvalue;
		}
	    }
	}
    }
      
  /**********************
   * Cleanup
   **********************/
  
  delete outarraydata;
  delete arraydata;
  dataSetServer->close(outset);
  dataSetServer->close(set);
      

  ENDPROC;
}
