
/* Sas-specific */
#include "Tracer.h"
#include "errstr.h"
#include "Dal.h"

/* Local */

int main ()
{
  PROC;

  string corrarrayname  = "a.ds:corr";
  string rawarrayname   = "a.ds:raw";
  string exparrayname   = "a.ds:exp";

  unsigned dx = 10;
  unsigned dy = 10;

  longVec size;
  size.push_back(dy);
  size.push_back(dx);
  
  /** Corrected image */

  errstr << "Opening " << corrarrayname << " .."
	 << Sas::message(Msg::App, Msg::Sparse);
  DataSet * corrset  = dataSetServer->open(corrarrayname, Dal::Create);
  Array * corrarray  = corrset->addArray(corrarrayname,
				       Dal::Real32,
				       size);
  ArrayData * corrarraydata = corrarray->data();
  real32 * corrdata = corrarraydata->real32Data();

  for (unsigned x = 0; x < dx; ++x)
    {
      for (unsigned y = 0; y < dy; ++y)
	{
	  corrdata[x+(y*dx)] = 1;
	}
    }

  delete corrarraydata;
  dataSetServer->close(corrset);

  /** Raw image */

  errstr << "Opening " << rawarrayname << " .."
	 << Sas::message(Msg::App, Msg::Sparse);

  /* Have to perform some manual checking here in case
     the dataset specified through rawarrayname is the same
     as that for the corrected image. 
     If so, don't create a new dataset, just modify it.
  */
  
  DataSet * rawset = 0;
  if (dataSetServer->exists(rawarrayname))
    {
      rawset  = dataSetServer->open(rawarrayname, Dal::Modify);
    }
    else
    {
      rawset  = dataSetServer->open(rawarrayname, Dal::Create);
    }

  if (!rawset)
    {
      errstr << "DAL entity " << rawarrayname
	     << " could not be created. Raise an SPR "
	     << Sas::error("invalidDalEntity");
    }
      
  Array * rawarray  = rawset->addArray(rawarrayname,
				       Dal::Int32,
				       size);
  ArrayData * rawarraydata = rawarray->data();
  int32 * rawdata = rawarraydata->int32Data();

  for (unsigned x = 0; x < dx; ++x)
    {
      for (unsigned y = 0; y < dy; ++y)
	{
	  rawdata[x+(y*dx)] = 1;
	}
    }
  delete rawarraydata;
  dataSetServer->close(rawset);


  /** Exposure map */
  errstr << "Opening " << exparrayname << " .."
	 << Sas::message(Msg::App, Msg::Sparse);
  DataSet * expset = 0;
  if (dataSetServer->exists(exparrayname))
    {
      expset  = dataSetServer->open(exparrayname, Dal::Modify);
    }
    else
    {
      expset  = dataSetServer->open(exparrayname, Dal::Create);
    }

  if (!expset)
    {
      errstr << "DAL entity " << exparrayname
	     << " could not be created. Raise an SPR "
	     << Sas::error("invalidDalEntity");
    }
  Array * exparray  = expset->addArray(exparrayname,
				       Dal::Real32,
				       size);
  ArrayData * exparraydata = exparray->data();
  real32 * expdata = exparraydata->real32Data();

  for (unsigned x = 0; x < dx; ++x)
    {
      for (unsigned y = 0; y < dy; ++y)
	{
	  expdata[x+(y*dx)] = 1;
	}
    }

  delete exparraydata;
  dataSetServer->close(expset);


  ENDPROC;
}
